<?php

defined('ABSPATH') || exit;

// Add settings menu
add_action('admin_menu', function () {
	add_options_page(
		'Purple Leaf Settings',
		'Purple Leaf',
		'manage_options',
		'purple-leaf',
		'purple_leaf_render_settings_page'
	);
});

// Register setting and field
add_action('admin_init', function () {
	register_setting(
		'purple_leaf_settings',
		'purple_leaf_connection_key',
		'purple_leaf_sanitize_connection_key'
	);
});

add_action('admin_enqueue_scripts', 'purple_leaf_admin_enqueue_scripts');

function purple_leaf_admin_enqueue_scripts($hook) {
	// Only load on the Purple Leaf settings page
	if ($hook !== 'settings_page_purple-leaf') {
		return;
	}

	// Register a dummy script handle (required for wp_add_inline_script)
	wp_register_script('purple-leaf-settings', '', [], false, true);

	wp_enqueue_script('purple-leaf-settings');

	wp_add_inline_script(
		'purple-leaf-settings',
		'document.addEventListener("DOMContentLoaded", function () {
			const form = document.querySelector("form");
			const textarea = document.getElementById("purple_leaf_connection_key");
			const submitBtn = form.querySelector("input[type=\'submit\']");
			form.addEventListener("submit", function (e) {
				if (!textarea.value.trim()) {
					e.preventDefault();
					alert("Please enter your connection key before saving.");
					textarea.focus();
					return;
				}
				submitBtn.disabled = true;
				submitBtn.value = "Validating...";
			});
		});'
	);
}

function purple_leaf_sanitize_connection_key($input) {
	return purple_leaf_validate_connection_key($input);
}

// Render settings page
function purple_leaf_render_settings_page() {
	$connection_key = get_option('purple_leaf_connection_key');
	wp_cache_delete('purple_leaf_is_connected', 'options');
	$is_connected = get_option('purple_leaf_is_connected', false);
	?>
	<div class="wrap">
		<h1>Purple Leaf Settings</h1>
		<?php if ($is_connected): ?>
            <div class="notice notice-success is-dismissible">
                <p>✅ This site is connected to Purple Leaf.</p>
            </div>
		<?php endif; ?>
		<form method="post" action="options.php">
			<?php settings_fields('purple_leaf_settings'); ?>
			<div style="max-width: 600px; margin-bottom: -16px">
				<label for="purple_leaf_connection_key" style="display:block; font-weight:bold; margin-bottom:6px;">
					Your Purple Leaf Connection Key
				</label>
				<textarea
					id="purple_leaf_connection_key"
					name="purple_leaf_connection_key"
					rows="4"
					class="large-text code"
					style="width: 100%;"
				><?php echo esc_textarea($connection_key); ?></textarea>
			</div>
            <div style="margin-top: 16px;">
				<?php submit_button('Connect to Purple Leaf'); ?>
                <p style="margin-top: -12px; color: #555;">
                    This will validate your connection key and activate the plugin.
                </p>
                <p style="margin-top: 4px; font-size: 13px;">
                    To learn more about how you can connect this WordPress site to Purple Leaf, <a href="https://purpleleaf.ai/how-to/connect-a-wordpress-site-to-purple-leaf" target="_blank">click here</a>.
                </p>
            </div>
        </form>
	</div>
	<?php
}

/**
 * Validate and save the Purple Leaf connection key.
 *
 * @param string $input
 * @return string Existing or new validated key
 */
function purple_leaf_validate_connection_key($input) {
	static $already_validated = false;
	if ($already_validated) {
		return $input;
	}
	$already_validated = true;

    if (!current_user_can('manage_options')) {
		return get_option('purple_leaf_connection_key');
	}

	if (
		!isset($_POST['_wpnonce']) ||
		!wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['_wpnonce'])), 'purple_leaf_settings-options')
	) {
		return get_option('purple_leaf_connection_key');
	}

	$trimmed = sanitize_text_field(trim($input));

    // Check if key is not empty
	if (empty($trimmed)) {
		add_settings_error(
			'purple_leaf_connection_key',
			'purple_leaf_connection_key_empty',
			'Connection key cannot be empty.',
			'error'
		);
		return get_option('purple_leaf_connection_key'); // Preserve old value
	}

	// Check if key is properly formatted
	if (!purple_leaf_is_valid_base64($trimmed)) {
		add_settings_error(
			'purple_leaf_connection_key',
			'purple_leaf_connection_key_format_error',
			'The connection key must be a valid base64-encoded string.',
			'error'
		);
		return get_option('purple_leaf_connection_key');
	}

    // Get endpoint
	$endpoint = purple_leaf_get_endpoint();

    // Check endpoint
	if (empty($endpoint)) {
		add_settings_error(
			'purple_leaf_connection_key',
			'purple_leaf_endpoint_missing',
			'Configuration error: Endpoint is missing or invalid in config.json.',
			'error'
		);
		return get_option('purple_leaf_connection_key');
	}

	$response = wp_remote_post($endpoint . 'ActivateConnection', [
		'headers' => [
                'Content-Type'          => 'application/json',
                'x-ppl-plugin-type'     => 'WordPress',
            ],
		'body'    => wp_json_encode(['key' => $trimmed, 'url' => get_site_url()]),
		'timeout' => 10,
	]);

	if (is_wp_error($response)) {
		add_settings_error(
			'purple_leaf_connection_key',
			'purple_leaf_connection_key_http_error',
			'Could not validate connection key: ' . $response->get_error_message(),
			'error'
		);
		return get_option('purple_leaf_connection_key');
	}

	$code = wp_remote_retrieve_response_code($response);
	$body = json_decode(wp_remote_retrieve_body($response), true);

	if ($code !== 200 || empty($body['url'])) {
		add_settings_error(
			'purple_leaf_connection_key',
			'purple_leaf_connection_key_invalid',
			'The provided connection key is invalid' . (!empty($body['errorCode']) ? ' (' . esc_html($body['errorCode']) . ').' : '.'),
			'error'
		);
		return get_option('purple_leaf_connection_key');
	}
	add_settings_error(
		'purple_leaf_connection_key',
		'purple_leaf_connection_key_success',
		'Your connection key has been validated and saved successfully. The Purple Leaf plugin is now active.',
		'updated'
	);
	add_option('purple_leaf_is_connected', true);
	return $trimmed;
}

// Show warning if Connection key is missing
add_action('admin_notices', function () {
	if (!current_user_can('manage_options')) {
		return;
	}

	// Show "missing key" warning if not connected
	if (!get_option('purple_leaf_connection_key')) {
		echo '<div class="notice notice-warning is-dismissible">
			<p><strong>Purple Leaf:</strong> Please enter your connection key in the space provided below to connect this WordPress site with Purple Leaf.</p>
		</div>';
	}

	// Show one-time notices (like deactivated success)
	$status = get_transient('purple_leaf_status_notice');
	if (!$status) return;

	$parts = explode(':', $status);
	$action = $parts[0] ?? 'unknown';
	$result = $parts[1] ?? 'unknown';

	if ($result === 'success') {
		echo '<div class="notice notice-success is-dismissible">
			<p><strong>Purple Leaf:</strong> Plugin ' . esc_html($action) . ' successfully.</p>
		</div>';
	}

	// Cleanup: delete after showing
	delete_transient('purple_leaf_status_notice');
});