<?php
/*
Plugin Name: Purple Leaf
Plugin URI: https://purpleleaf.ai
Description: Automatically generate social media posts for your WordPress site.
Version: 1.0
Author: gmantri
Author URI: https://profiles.wordpress.org/gmantri/
License: GPLv2 or later
*/

defined('ABSPATH') || exit;

require_once plugin_dir_path(__FILE__) . 'utils.php';
require_once plugin_dir_path(__FILE__) . 'request.php';
require_once plugin_dir_path(__FILE__) . 'admin.php';

// plugin activation hook
register_activation_hook(__FILE__, function () {
	add_option('purple_leaf_activation_redirect', true);
});

// plugin deactivation hook
register_deactivation_hook(__FILE__, function () {
	$response = purple_leaf_post_to_api('DeactivateConnection', []);
	if ($response['success']) {
		update_option('purple_leaf_status_notice', 'deactivated:success', 60);
	} else {
		delete_option('purple_leaf_status_notice');
	}
	delete_option('purple_leaf_connection_key');
	delete_option('purple_leaf_is_connected');
	wp_cache_delete('purple_leaf_is_connected', 'options');
});

// handle post transition
add_action('transition_post_status', 'purple_leaf_handle_post_status_change', 10, 3);

// handle slug changes in save post
add_action('save_post', 'purple_leaf_handle_slug_change', 20, 3);

// handle permanent post deletion
add_action('before_delete_post', 'purple_leaf_handle_permanent_deletion');

// handle admin init
add_action('admin_init', function () {
	if (get_option('purple_leaf_activation_redirect', false)) {
		delete_option('purple_leaf_activation_redirect');

		if (!filter_input(INPUT_GET, 'activate-multi')) {
			wp_safe_redirect(admin_url('options-general.php?page=purple-leaf'));
			exit;
		}
	}
});

// handle admin notices
add_action('admin_notices', function () {
	$status = get_option('purple_leaf_status_notice');
	if (!$status) return;

	$parts = explode(':', $status);
	$action = $parts[0] ?? 'unknown';
	$result = $parts[1] ?? 'unknown';

	if ($result === 'success') {
		echo '<div class="notice notice-success is-dismissible">
			<p><strong>Purple Leaf:</strong> Plugin ' . esc_html($action) . ' successfully.</p>
		</div>';
	}

	delete_option('purple_leaf_status_notice');
});

// post status change handler
function purple_leaf_handle_post_status_change($new_status, $old_status, $post) {
	if (!in_array($post->post_type, purple_leaf_supported_post_types())) return;
	if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) return;
	if (wp_is_post_revision($post->ID)) return;

	if (purple_leaf_should_debounce($post->ID)) return;

	// get the permalink before it changes
	$url = purple_leaf_get_stable_permalink($post, $old_status, $new_status);

	if ($old_status === 'publish' && $new_status !== 'publish') {
		purple_leaf_send_webhook('deleted', $post, $url);
		return;
	}

	// handle post publish (exclude scheduled posts as they are dealt separately)
	if ($old_status !== 'publish' && $old_status !== 'future' && $new_status === 'publish') {
		purple_leaf_send_webhook('created', $post, $url);
		return;
	}

	// handle scheduled posts
	if ($old_status === 'future' && $new_status === 'publish') {
		purple_leaf_send_webhook('created', $post, $url);
		return;
	}

	// handle updates
	if ($old_status === 'publish' && $new_status === 'publish') {
		$revisions = wp_get_post_revisions($post->ID);
		$last_revision = reset($revisions);

		if ($last_revision && !purple_leaf_post_has_changed($last_revision, $post)) return;
		purple_leaf_send_webhook('updated', $post, $url);
		return;
	}

	// handle soft deletes
	if ($new_status === 'trash' && $old_status === 'publish') {
		purple_leaf_send_webhook('deleted', $post, $url);
		return;
	}

	// handle undelete
	if ($old_status === 'trash' && $new_status === 'publish') {
		purple_leaf_send_webhook('updated', $post, $url);
		return;
	}
}

// post permanent deletion handler
function purple_leaf_handle_permanent_deletion($post_id) {
	$post = get_post($post_id);
	// get the permalink before it changes
	$url = get_permalink($post);

	if (!$post || !in_array($post->post_type, purple_leaf_supported_post_types())) {
		return;
	}

	if (!get_option('purple_leaf_is_connected')) {
		return;
	}

	purple_leaf_send_webhook('deleted', $post, $url);
}

function purple_leaf_handle_slug_change($post_id, $post, $update) {
	if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) return;
	if (!in_array($post->post_type, purple_leaf_supported_post_types())) return;
	if ($post->post_status !== 'publish') return; // 🔒 only track public slugs
	if (wp_is_post_revision($post_id)) return;

	$old_slug = get_post_meta($post_id, '_purple_leaf_previous_slug', true);
	$current_slug = $post->post_name;

	if ($old_slug && $old_slug !== $current_slug) {
		$old_url = home_url("/$old_slug/");
		$new_url = get_permalink($post);

		// send delete webhook for old URL
		purple_leaf_send_webhook('deleted', $post, $old_url);

		// send created webhook for new URL
		purple_leaf_send_webhook('created', $post, $new_url);
	}

	// always store the current slug for future comparison
	update_post_meta($post_id, '_purple_leaf_previous_slug', $current_slug);
}

// utility function to invoke the api
function purple_leaf_send_webhook($event_type, $post, $url) {
	$payload = [
		'action'    => $event_type, // 'created', 'updated', or 'deleted'
		'id'      => $post->ID,
		'title'   => get_the_title($post),
		'url'     => $url,
		'status'  => $post->post_status,
		'typeOf'  => $post->post_type,
		'updated' => $post->post_modified_gmt,
	];

	purple_leaf_post_to_api('ProcessPage', $payload);
}