<?php

defined('ABSPATH') || exit;

/**
 * Send a POST request to the Purple Leaf API.
 *
 * @param string $path        API path (e.g., 'ActivatePlugin')
 * @param array  $payload     JSON payload to send
 * @param int    $retry       Number of retry attempts on failure
 * @return array|false        Associative array on success/failure or false if precondition fails
 */
function purple_leaf_post_to_api($path, $payload = [], $retry = 5) {
	$is_connected = get_option('purple_leaf_is_connected', false);
	if (!$is_connected) {
		return [
			'success' => false,
			'code'    => -1,
			'body'    => ['message' => 'Plugin not connected or validated.'],
		];
	}
	$endpoint = function_exists('purple_leaf_get_endpoint') ? purple_leaf_get_endpoint() : null;
	if (empty($endpoint)) {
		purple_leaf_log_message('Purple Leaf: API call skipped - endpoint missing.');
		return false;
	}

	$connection_key = get_option('purple_leaf_connection_key');
	if (empty($connection_key)) {
		purple_leaf_log_message('Purple Leaf: API call skipped - connection key missing.');
		return false;
	}

	$main_plugin_file = plugin_dir_path(__FILE__) . 'purple-leaf.php';
	$plugin_data = get_file_data($main_plugin_file, ['Version' => 'Version']);
	$plugin_version = $plugin_data['Version'] ?? 'unknown';

	$url = $endpoint . ltrim($path, '/');

	$response = wp_remote_post($url, [
		'headers' => [
			'Content-Type'                => 'application/json',
			'x-ppl-plugin-connection-key' => $connection_key,
			'x-ppl-plugin-version'        => $plugin_version,
			'x-ppl-plugin-type'           => 'WordPress',
		],
		'body'    => wp_json_encode($payload),
		'timeout' => 10,
	]);

	// Handle WP errors (network issues, etc.)
	if (is_wp_error($response)) {
		$error_message = $response->get_error_message();
		purple_leaf_log_message("Purple Leaf API Error ($path): " . $error_message);

		// Retry for network-related errors
		if ($retry > 0 && (
				strpos($error_message, 'cURL error') !== false ||
				strpos($error_message, 'Connection timed out') !== false ||
				strpos($error_message, 'Could not resolve host') !== false
			)) {
			// Wait with exponential backoff (1s, 2s, 4s...)
			// This helps with transient issues like timeouts or server overload
			sleep(pow(2, 3 - $retry));
			return purple_leaf_post_to_api($path, $payload, $retry - 1);
		}

		return [
			'success' => false,
			'code'    => 0,
			'body'    => null,
		];
	}

	$code = wp_remote_retrieve_response_code($response);
	$body = json_decode(wp_remote_retrieve_body($response), true);

	if ($code !== 200) {
		$error_code = !empty($body['errorCode']) ? $body['errorCode'] : 'unknown';
		purple_leaf_log_message("Purple Leaf API Error ($path): HTTP $code (Error Code: $error_code)");

		// Only retry for server errors (5xx), not client errors (4xx)
		if ($retry > 0 && $code >= 500) {
			// Wait with exponential backoff (1s, 2s, 4s...)
			// This helps with transient issues like timeouts or server overload
			sleep(pow(2, 5 - $retry));
			return purple_leaf_post_to_api($path, $payload, $retry - 1);
		}

		return [
			'success' => false,
			'code'    => $code,
			'body'    => $body,
		];
	}

	return [
		'success' => true,
		'code'    => $code,
		'body'    => $body,
	];
}