<?php

defined('ABSPATH') || exit;

/**
 * Log a message to the error log if WP_DEBUG and WP_DEBUG_LOG are enabled.
 *
 * @param mixed $message String, array, or object to log.
 * @return void
 */
function purple_leaf_log_message($message): void {
	if (!defined('WP_DEBUG') || !WP_DEBUG || !defined('WP_DEBUG_LOG') || !WP_DEBUG_LOG) {
		return;
	}

	if (is_array($message) || is_object($message)) {
		$message = wp_json_encode($message, JSON_PRETTY_PRINT);
	}

	error_log('[Purple Leaf] ' . $message);
}

/**
 * Get API endpoint from config.json.
 *
 * @return string|null
 */
function purple_leaf_get_endpoint(): ?string {
	static $cached_endpoint = null;
	if ($cached_endpoint !== null) {
		return $cached_endpoint;
	}

	$config_path = plugin_dir_path(__FILE__) . 'config.json';

	if (!file_exists($config_path)) {
		purple_leaf_log_message('Purple Leaf: config.json not found at ' . $config_path);
		return null;
	}

	$config = json_decode(file_get_contents($config_path), true);

	if (!is_array($config) || empty($config['endpoint'])) {
		purple_leaf_log_message('Purple Leaf: Invalid or missing "endpoint" in config.json');
		return null;
	}

	$endpoint = sanitize_text_field($config['endpoint']);

	if (!filter_var($endpoint, FILTER_VALIDATE_URL)) {
		purple_leaf_log_message('Purple Leaf: Invalid endpoint URL in config.json: ' . $endpoint);
		return null;
	}

	return $cached_endpoint = trailingslashit($endpoint);
}

/**
 * Validate base64-encoded string.
 *
 * @param string $string
 * @return bool
 */
function purple_leaf_is_valid_base64($string): bool {
	// Only allow valid base64 characters
	if (!preg_match('/^[a-zA-Z0-9\/\r\n+]*={0,2}$/', $string)) {
		return false;
	}

	// Attempt strict decode
	$decoded = base64_decode($string, true);
	if ($decoded === false) {
		return false;
	}

	// Make sure re-encoding gives us the same string (ignoring newlines)
	return base64_encode($decoded) === str_replace(["\r", "\n"], '', $string);
}

/**
 * Compare old and new post objects for changes.
 *
 * @param WP_Post $old
 * @param WP_Post $new
 * @return bool
 */
function purple_leaf_post_has_changed($old, $new): bool {
	$old_thumbnail_id = get_post_thumbnail_id($old->ID);
	$new_thumbnail_id = get_post_thumbnail_id($new->ID);

	return (
		$old->post_title   !== $new->post_title ||
		$old->post_content !== $new->post_content ||
		$old->post_name !== $new->post_name ||
		$old_thumbnail_id  !== $new_thumbnail_id
	);
}

/**
 * Get a reliable permalink, even if post is trashed.
 *
 * @param WP_Post $post
 * @param string|null $old_status
 * @param string|null $new_status
 * @return string|null
 */
function purple_leaf_get_stable_permalink($post, $old_status = null, $new_status = null): ?string {
	if (!$post) return null;

	// If it's not being trashed, return the normal permalink
	if (!($old_status === 'publish' && ($new_status === 'trash' || $new_status === 'draft'))) {
		return get_permalink($post) ?? null;
	}

	// Special handling for trash: get full page URI (supports hierarchy)
	$uri = get_page_uri($post);
	if (!$uri) {
		return null;
	}
	// If trashed, the URI ends with __trashed — remove it
	if (str_ends_with($uri, '__trashed')) {
		$uri = substr($uri, 0, -9);
	}

	// Construct the proper URL with home_url and full URI
	return trailingslashit(home_url("/$uri"));
}

/**
 * Get supported post types.
 *
 * @return array
 */
function purple_leaf_supported_post_types(): array {
	return ['post', 'page'];
}

/**
 * Compares the current time with last webhook invocation time and returns true/false if
 * last time webhook invoked was 5 seconds ago.
 *
 * @param $post_id
 * @param int $threshold_seconds
 *
 * @return bool
 */
function purple_leaf_should_debounce($post_id, $threshold_seconds = 5): bool {
	$last_sent = absint(get_post_meta($post_id, '_purple_leaf_last_webhook_time', true));
	$now = time();

	if ($now - $last_sent < $threshold_seconds) {
		return true; // debounce — too soon since last event
	}

	update_post_meta($post_id, '_purple_leaf_last_webhook_time', $now);
	return false; // go ahead
}
